"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var nativescript_appversion_1 = require("nativescript-appversion");
var platform = require("tns-core-modules/platform");
var http = require("tns-core-modules/http");
var application_settings_1 = require("tns-core-modules/application-settings");
var moment = require("moment/moment");
var types_1 = require("tns-core-modules/utils/types");
var Common = (function () {
    function Common() {
        var _this = this;
        this.maxAmountOfBreadcrumbs = 50;
        this.storageKey = 'sentryCrashesQueue';
        this.breadcrumbs = [];
        this.versionName = '';
        this.versionCode = '';
        this.sentryId = '';
        this.environment = 'DEVELOPMENT';
        this.userId = 'unknown';
        if (Common.getInstance()) {
            throw new Error("Error: Instance failed: Use LiveEngage.getInstance() instead of new.");
        }
        Common.instance = this;
        setTimeout(function () {
            _this.setVersionName();
            _this.setVersionCode();
        });
    }
    Common.getInstance = function () {
        return Common.instance;
    };
    Common.prototype.setCredentials = function (sentryId, sentryKey, environment, userId) {
        this.sentryId = sentryId;
        this.endPoint = 'https://sentry.io/api/' + sentryId + '/store/?sentry_version=7&sentry_client=raven-js%2F3.22.2&sentry_key=' + sentryKey;
        if (environment) {
            this.environment = environment;
        }
        if (userId) {
            this.userId = userId;
        }
    };
    Common.prototype.setMaxAmountOfBreadcrumbs = function (amount) {
        this.maxAmountOfBreadcrumbs = amount;
    };
    Common.prototype.saveBreadcrumb = function (message, category, properties) {
        var crumb = {
            timestamp: moment(),
            message: message,
            category: category,
            data: properties
        };
        this.breadcrumbs.push(crumb);
        if (this.breadcrumbs.length > this.maxAmountOfBreadcrumbs) {
            this.breadcrumbs.shift();
        }
    };
    Common.prototype.saveCrash = function (errorMessage, errorDetails) {
        this.saveBreadcrumb('' + errorDetails, 'crash start');
        var deviceType = '';
        var language = '';
        var manufacturer = '';
        var model = '';
        var os = '';
        var osVersion = '';
        if (!types_1.isNullOrUndefined(platform.device)) {
            deviceType = platform.device.deviceType;
            language = platform.device.language;
            manufacturer = platform.device.manufacturer;
            model = platform.device.model;
            os = platform.device.os;
            osVersion = platform.device.osVersion;
        }
        var crashData = {
            project: this.sentryId,
            logger: 'nativescript',
            platform: 'javascript',
            message: '' + errorMessage,
            level: 'error',
            tags: {
                device_type: deviceType,
                device_lang: language,
                app_version: this.versionCode,
                trace_category: 'Error'
            },
            breadcrumbs: {
                values: this.breadcrumbs
            },
            environment: this.environment,
            release: this.versionName,
            contexts: {
                device: {
                    family: manufacturer,
                    model: model
                },
                os: {
                    name: os,
                    version: osVersion
                },
                runtime: {
                    name: 'Nativescript'
                }
            },
            user: {
                id: this.userId,
            }
        };
        var crashes = [];
        if (this.isPresentInSettings(this.storageKey)) {
            crashes = JSON.parse(application_settings_1.getString(this.storageKey));
        }
        crashes.push(crashData);
        application_settings_1.setString(this.storageKey, JSON.stringify(crashes));
    };
    Common.prototype.sendCrashes = function () {
        if (!this.isPresentInSettings(this.storageKey)) {
            return;
        }
        var crashes = JSON.parse(application_settings_1.getString(this.storageKey));
        for (var _i = 0, crashes_1 = crashes; _i < crashes_1.length; _i++) {
            var crash = crashes_1[_i];
            this.submitCrash(crash);
        }
    };
    Common.prototype.submitCrash = function (crash) {
        var _this = this;
        http.request({
            url: this.endPoint,
            method: "POST",
            headers: {
                "Content-Type": "application/json",
                'Accept': '*/*',
                'Origin': 'nativescript://'
            },
            content: JSON.stringify(crash)
        }).then(function (response) {
            var crashes = JSON.parse(application_settings_1.getString(_this.storageKey));
            var index = _this.customIndexOf(crashes, crash);
            if (index > -1) {
                crashes.splice(index, 1);
                application_settings_1.setString(_this.storageKey, JSON.stringify(crashes));
            }
        }, function (e) {
            console.error("Sentry send error", e);
        });
    };
    Common.prototype.setVersionName = function () {
        var _this = this;
        nativescript_appversion_1.getVersionName().then(function (versionName) {
            _this.versionName = versionName;
        });
    };
    Common.prototype.setVersionCode = function () {
        var _this = this;
        nativescript_appversion_1.getVersionCode().then(function (versionCode) {
            _this.versionCode = versionCode;
        });
    };
    Common.prototype.isPresentInSettings = function (storeKey) {
        return application_settings_1.getString(storeKey) !== undefined;
    };
    Common.prototype.customIndexOf = function (arr, searchElement) {
        for (var i = 0; i < arr.length; i++) {
            if (arr[i].message === searchElement.message) {
                return i;
            }
        }
        return -1;
    };
    return Common;
}());
Common.instance = new Common();
exports.Common = Common;
//# sourceMappingURL=ng-sentry.common.js.map